module Tyntec
  module REST
    # Rest client for communication with api endpoints
    # Could be configured by api key and logger or use global configuration
    #
    # Example:
    # client = Tyntec::REST::Client.new('api_key')
    # client.messaging.sms.send('+700000000', 'Message', 'Sender name')
    class Client
      attr_accessor :api_key

      include UriHelper

      TYNTEC_API_URI = 'https://api.tyntec.com'.freeze

      def initialize(api_key = nil, logger = nil)
        @api_key = api_key || Tyntec.api_key
        @logger = logger || Tyntec.logger
      end

      def messaging
        @messaging ||= Messaging.new self
      end

      def request(method, uri, params = {})
        raise ClientError 'Tyntec client not configured' unless configured?

        target_uri = absolute_url(uri)

        begin
          result = send_request(method, target_uri, params)
        rescue RestClient::ExceptionWithResponse => e
          @logger&.error(e)
          raise RequestError, e
        end

        JSON.parse(result)
      end

      protected

      def configured?
        !api_key.nil?
      end

      def absolute_url(uri)
        build_uri(TYNTEC_API_URI, uri)
      end

      def send_request(method, uri, params)
        case method.to_sym
        when :post
          post(uri, params)
        when :get
          get(uri, params)
        else
          raise ClientError, "Unknown method #{method}"
        end
      end

      def post(uri, params)
        RestClient.post(uri, params.to_json, headers)
      end

      def get(uri, params)
        RestClient.get(uri, params: params, headers: headers)
      end

      def headers
        {
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'apikey' => @api_key
        }
      end
    end
  end
end
